unit UnidadMenuPrincipal;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ShellCtrls, OleServer, CmAdmCtl, Menus,
  ActnList, UnidadOpciones, ShellAPI;

const
  MyKey  = '\Software\AjpdSoft\InfoCar';
  WM_THREAD_DONE = WM_USER + $100;
  BackColor      = '"#FFFFFF"';
  HiddenColor    = '"#ccccff"';
  ReadOnlyColor  = '"#ffccff"';
  NormalColor    = '"#ccffff"';

type

  PThreadData = ^TThreadData;
  TThreadData = record
    Path       : string;
    Terminated : boolean;
    ROH        : boolean;
    ScanFiles  : boolean;
  end;

  TformMenuPrincipal = class(TForm)
    sbMain: TStatusBar;
    shlTree: TShellTreeView;
    mnuMain: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    SavetoHTML1: TMenuItem;
    Preferences1: TMenuItem;
    N1: TMenuItem;
    ActionList1: TActionList;
    ExportarHTML: TAction;
    Cancel: TAction;
    saveDlg: TSaveDialog;
    Options: TAction;
    Exit: TAction;
    About: TAction;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure shlTreeChange(Sender: TObject; Node: TTreeNode);
    procedure sbMainResize(Sender: TObject);
    procedure ExportarHTMLExecute(Sender: TObject);
    procedure CancelExecute(Sender: TObject);
    procedure shlTreeKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure OptionsExecute(Sender: TObject);
    procedure ExitExecute(Sender: TObject);
    procedure AboutExecute(Sender: TObject);
    procedure sbMainDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel;
      const Rect: TRect);
  private
    FSettings  : TSettings;
    FPrintRoot : boolean;
    FCancel    : boolean;
    FFldCount  : longint;
    FFilesCount: longint;
    FileHandle : TextFile;
    FileName   : string;
    TH    : THandle;
    TID   : cardinal;
    Data  : TThreadData;
    procedure FindFiles(const Path : string;var Terminated : boolean);
    procedure DoSearch(const Path : string;var Terminated : boolean);
    procedure ThreadDone(var M : TMessage);message WM_THREAD_DONE;
    procedure LoadSettings;
    procedure SaveSettings;
    function IsKeyAvailable:boolean;
    procedure OnMyIdle(Sender: TObject; var Done: Boolean);

    procedure WriteAuthor;
    procedure WriteSummary;
    procedure WriteHeader;
    procedure WriteFooter;
    procedure WriteLegend;
    procedure BeginList(Indented : boolean);
    procedure CloseList(Indented : boolean);
    procedure PrintItem(Value : string);
    function GetFileColors(const SR : TSearchRec):string;
    procedure PrintFile(const SR : TSearchRec);
    procedure OpenHTMLFile(const FileName : string);
    procedure CloseHTMLFile();
    procedure BeginTable();
    procedure CloseTable();
    procedure PrintCount(const Desc : string;Count : longint);
  public
    { Public declarations }
  end;

  function SearchThread(P : pointer):DWord;stdcall;

var
  formMenuPrincipal: TformMenuPrincipal;

implementation

uses Registry, UnidadAcercaDe;

{$R *.dfm}

procedure TformMenuPrincipal.OnMyIdle(Sender: TObject; var Done: Boolean);
begin
  sbMain.Panels[2].Text := TimeToStr(Now);
end;

procedure TformMenuPrincipal.FindFiles(const Path: string;var Terminated : boolean);
var
  SR : TSearchRec;
  i  : longint;
  Attr : integer;
  NewPath : string;
begin
  i := 0;
  NewPath := Path;
//  if Length(NewPath) > MAX_PATH then NewPath := '\\?\'+NewPath;
  if FSettings.ROH then
    Attr := faArchive+faReadOnly+faSysFile+faHidden+faVolumeID+faAnyFile
  else
    Attr := faArchive;

  if FindFirst(NewPath+'*.*',Attr ,SR)= 0 then
    try
      BeginTable;
      repeat
        Inc(i);
        PrintFile(SR);
      until (FindNext(SR) <> 0) or Terminated;
    finally
      PrintCount('Total ',i);
      FFilesCount := FFilesCount + i;
      CloseTable;
      SysUtils.FindClose(SR);
    end;
end;

procedure TformMenuPrincipal.DoSearch(const Path: string;var Terminated : boolean);
var
  SR : TSearchRec;
  Indent : boolean;
  Attr : integer;
begin
  Indent := false;
  if FPrintRoot and FSettings.ScanFiles then
    begin
      FPrintRoot := false;
      FindFiles(Path+'\'+SR.Name+'\',Terminated);
    end;

  if FSettings.ROH then
    Attr := faReadOnly+faSysFile+faHidden+faDirectory
  else
    Attr := faDirectory;

  if FindFirst(Path+'*.*',Attr,SR) = 0 then
    try
      Inc(FFldCount);
      BeginList(Indent);
      repeat
        if (((SR.Attr and faDirectory) <> 0) and (SR.Name[1] <> '.')) and
            not Terminated then
          begin
            PrintItem(SR.Name);
            if FSettings.ScanFiles then FindFiles(Path+'\'+SR.Name+'\',Terminated);
            DoSearch(Path+SR.Name+'\',Terminated);
          end;
      until (FindNext(SR) <> 0) or Terminated;
      CloseList(Indent);
    finally
      SysUtils.FindClose(SR);
    end;
end;

function SearchThread(P : pointer):DWord;
begin
  formmenuprincipal.FPrintRoot := true;
  formmenuprincipal.FCancel := false;
  formmenuprincipal.OpenHTMLFile(formmenuprincipal.FileName);
  formmenuprincipal.WriteHeader;
  formmenuprincipal.DoSearch(PThreadData(P)^.Path,PThreadData(P)^.Terminated);
  formmenuprincipal.WriteFooter;
  formmenuprincipal.CloseHTMLFile;
  SendMessage(formmenuprincipal.Handle,WM_THREAD_DONE,0,0);
  Result := 0;
end;

procedure TformMenuPrincipal.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Data.Terminated := true;
  if (TH <> 0) then CloseHandle(TH);
  FSettings.Width := Width;
  FSettings.Height:= Height;
  FSettings.Left  := Left;
  FSettings.Top   := Top;
  SaveSettings;
end;

procedure TformMenuPrincipal.BeginList(Indented: boolean);
begin
  WriteLn(FileHandle,'<ul type="square">');
end;

procedure TformMenuPrincipal.CloseList(Indented: boolean);
begin
  WriteLn(FileHandle,'</ul>');
end;

procedure TformMenuPrincipal.PrintItem(Value: string);
begin
  WriteLn(FileHandle,'<li><h4>'+Value+'</h4></li>');
end;

procedure TformMenuPrincipal.CloseHTMLFile;
begin
  CloseFile(FileHandle);
end;

procedure TformMenuPrincipal.OpenHTMLFile(const FileName: string);
begin
  AssignFile(FileHandle,FileName);
  Rewrite(FileHandle);
end;

procedure TformMenuPrincipal.BeginTable;
begin
  WriteLn(FileHandle,
  '<table bordercolor="silver" bgcolor="#ffffff" border="0" cellpadding="4" cellspacing="0">'+
  '<tr><td bgcolor="#33ccff"><font color="#ffffff" size="2"><b>Nombre del fichero</b></font></td>'+
  '<td bgcolor="#33ccff"><font color="#ffffff" size="2"><b>Tamao</b></font></td>'+
  '<td bgcolor="#33ccff"><font color="#ffffff" size="2"><b>Fecha ltima modificacin</b></font></td></tr>');
end;

procedure TformMenuPrincipal.CloseTable;
begin
  WriteLn(FileHandle,'</table><br>');
end;

procedure TformMenuPrincipal.shlTreeChange(Sender: TObject; Node: TTreeNode);
begin
  sbMain.Panels[1].Text := (Sender as TShellTreeView).SelectedFolder.PathName;
end;

procedure TformMenuPrincipal.sbMainResize(Sender: TObject);
begin
  (Sender as TStatusBar).Panels[1].Width := Self.Width - 120;
end;

procedure TformMenuPrincipal.WriteFooter;
begin
  if FSettings.Summ then WriteSummary;
  WriteLn(FileHandle,
  '</body></html>');
end;

procedure TformMenuPrincipal.WriteAuthor;
begin
  WriteLn(FileHandle,
  '<table>'+
  '<tr><td><font color="#000000" size="2">Fecha</font></td>'+
  '<td><font color="#000000" size="2">: '+ DateTimeToStr(Now)+'</font></td></tr>'+
  '<tr><td><font color="#000000" size="2">Generado por</font></td>'+
  '<td><font color="#000000" size="2">: '+ FSettings.AuthStr+'</td></tr></font></table><br>');
end;

procedure TformMenuPrincipal.WriteHeader;
begin

  WriteLn(FileHandle,
  '<html><head>'+
  '<title> Documento de '+FileName+'</title>'+
  '<style type="text/css">'+
  '<!--'+
  'body { background-color: #FFFFFF;'+
	'font-family: Arial,sans-serif;'+
	'color: #330066;'+
	'padding: 50px, 70px }'+
  'h1 { color: #000000; font-size: 20; font-weight: bold }'+
  'h2 { color: #ffffff; font-size: 12; font-weight: bold }'+
  'h3 { color: #f00000; font-size: 12; font-weight: normal }'+
  'h4 { color: #0000ff; font-size: 14; font-weight: bold}'+
  'h5 { color: #0000ff; font-size: 6; font-weight: normal}'+
  'p { font-family: Arial; color: #800080; font-size: 14px }'+
  '.CUSTOM { font-size: large; color: #FF8000 }'+
  '-->'+
  '</style>'+
  '</head>'+
  '<body><br>');
  if FSettings.Author then WriteAuthor;
  WriteLn(FileHandle,'<h1>Estructura de '+ Data.Path+'</h1><hr><br>');
  if FSettings.Legend then WriteLegend;

end;

procedure TformMenuPrincipal.ExportarHTMLExecute(Sender: TObject);
var
  Path : string;
begin
  Path := shlTree.SelectedFolder.PathName;
  if Path <> '' then
    begin
     if saveDlg.Execute then
       begin
         if Path[Length(Path)] <> '\' then
           Data.Path := Path+'\'
         else
           Data.Path := Path;
         FFilesCount := 0;
         FFldCount   := 0;
         FileName  := saveDlg.FileName;
         Data.Terminated := false;
         TH := CreateThread(nil,0,@SearchThread,Pointer(@Data),0,TID);
         sbMain.Panels[1].Text := 'Espere por favor... Pulse Escape para Cancelar el proceso...';
       end;
    end
  else
    MessageDlg('Debe seleccionar una carpeta.',mtInformation,[mbOK],0);
end;

function TformMenuPrincipal.GetFileColors(const SR: TSearchRec): string;
begin
  if (SR.Attr and faHidden) <> 0 then Result := HiddenColor
  else
  if (SR.Attr and faReadOnly) <> 0 then Result := ReadOnlyColor
  else
    Result := NormalColor;
end;

procedure TformMenuPrincipal.PrintFile(const SR: TSearchRec);
begin
  WriteLn(FileHandle,
  '<tr>'+
  '<td bgcolor='+GetFileColors(SR)+'><font color="#000000" size="1">'+SR.Name+'</font></td>'+
  '<td bgcolor='+GetFileColors(SR)+'><font color="#000000" size="1">'+IntToStr(SR.Size)+' bytes </font></td>'+
  '<td bgcolor='+GetFileColors(SR)+'><font color="#000000" size="1">'+DateTimeToStr(FileDateToDateTime(SR.Time))+'</font></td></tr>');
end;

procedure TformMenuPrincipal.PrintCount(const Desc: string; Count: Integer);
begin
  WriteLn(FileHandle,
  '<tr><td bgcolor="#ccffff"><font color="#ff0000" size="2">'+Desc+'</font></td>'+
  '<td bgcolor="#ccffff"><font color="#ff0000" size="2">'+IntToStr(Count)+' Ficheros </font></td>'+
  '<td bgcolor="#ccffff"><font color="#ff0000" size="2"></font></td>');
end;

procedure TformMenuPrincipal.CancelExecute(Sender: TObject);
begin
  Data.Terminated := true;
end;

procedure TformMenuPrincipal.shlTreeKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    begin
      Data.Terminated := true;
      FCancel := true;
    end;
end;

procedure TformMenuPrincipal.ThreadDone(var M: TMessage);
begin
  Data.Terminated := true;
  sbMain.Panels[1].Text := 'Proceso realizado';
  CloseHandle(TH);
  if not FCancel then  ShellExecute(Handle,'open',PChar(FileName),nil,nil,SW_SHOWNORMAL);
  M.Result := 0;
end;

procedure TformMenuPrincipal.WriteSummary;
begin
  WriteLn(FileHandle,
  '<hr><table width="20%" cellpadding="2" cellspacing="0" border="0" bordercolor="silver" bgcolor='+BackColor+'>'+
  '<tr><td  colspan="2"><font color="#000000" size=1><b>Resumen</b></font></td></tr>'+
  '<tr><td><font color="#000000" size=1>Total carpetas</font></td><td><font color="#000000" size=1>'+
  IntToStr(FFldCount-1)+'</font></td></tr>'+
  '<tr><td><font color="#000000" size=1>Total Ficheros</font></td><td><font color="#000000" size=1>'+
  IntToStr(FFilesCount)+'</font></td></tr></table><hr><br>'+
  '<h6>Todos los derechos de copia reservados&nbsp;&copy;&nbsp;2004 AjpdSoft <a href="www.ajpdsoft.com">'+
  'AjpdSoft</a></h6>');

end;

procedure TformMenuPrincipal.WriteLegend;
begin
  WriteLn(FileHandle,
  '<table width="20%" cellpadding="2" cellspacing="0" border="1" bordercolor="silver" bgcolor='+BackColor+'>'+
  '<tr><td  colspan="2"><font color="#000000" size=1><b>Leyenda</b></font></td></tr>'+
  '<tr><td><font color="#000000" size=1>Ficheros</font></td><td bgcolor='+NormalColor+'>&nbsp;&nbsp;</td></tr>'+
  '<tr><td><font color="#000000" size=1>Ficheros de slo lectura</font></td><td bgcolor='+ReadOnlyColor+'>&nbsp;&nbsp;</td></tr>'+
  '<tr><td><font color="#000000" size=1>Ficheros ocultos</font></td><td bgcolor='+HiddenColor+'>&nbsp;&nbsp;</td></tr></table><br>');
end;

procedure TformMenuPrincipal.LoadSettings;
var
	Reg  : TRegistry;
begin
	Reg := TRegistry.Create;
    try
      Reg.RootKey := HKEY_LOCAL_MACHINE;
      Reg.OpenKey(MyKey,true);
      with FSettings,Reg do
        begin
          ROH   := ReadBool('ReadOnly');
          ScanFiles := ReadBool('ReadFiles');
          Summ  := ReadBool('Summary');
          Legend:= ReadBool('Legend');
          Author:= ReadBool('Author');
          Width := ReadInteger('Width');
          Height:= ReadInteger('Height');
          Left  := ReadInteger('Left');
          Top   := ReadInteger('Top');
        end;

    finally
      Reg.CloseKey;
      Reg.Free;
    end;
end;

procedure TformMenuPrincipal.SaveSettings;
var
	Reg  : TRegistry;
begin
	Reg := TRegistry.Create;
    try
      Reg.RootKey := HKEY_LOCAL_MACHINE;
      Reg.OpenKey(MyKey,true);
      with FSettings,Reg do
        begin
          WriteBool('ReadOnly',ROH);
          WriteBool('ReadFiles',ScanFiles);
          WriteBool('Summary',Summ);
          WriteBool('Legend',Legend);
          WriteBool('Author',Author);
          WriteInteger('Width',Self.Width);
          WriteInteger('Height',Self.Height);
          WriteInteger('Left',Self.Left);
          WriteInteger('Top',Self.Top);
        end;
    finally
      Reg.CloseKey;
      Reg.Free;
    end;
end;

function TformMenuPrincipal.IsKeyAvailable: boolean;
var
	Reg : TRegistry;
begin
	Reg  := TRegistry.Create;
    try
  	  Reg.RootKey  := HKEY_LOCAL_MACHINE;
	  Result := Reg.OpenKey(MyKey,false);
    finally
      Reg.CloseKey;
      Reg.Free;
    end;
end;

procedure TformMenuPrincipal.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  {process is running}
  if not Data.Terminated then
    begin
      if MessageDlg('El proceso no ha finalizado, '+#13#10+'Desea cancelarel proceso?',
                    mtInformation,[mbYes,mbNo],0)= mrYes then
        begin
          CanClose := true;
        end;
    end;
end;

procedure TformMenuPrincipal.FormCreate(Sender: TObject);
var
  Size : Cardinal;
begin
  Application.OnIdle := OnMyIdle;
  Data.Terminated := true;
  SetLength(FSettings.AuthStr,255);
  FillChar(FSettings.AuthStr,255,#0);
  size := 255;
  try
    GetUserName(PChar(FSettings.AuthStr),size);
  except
    //silenciosa
  end;
  if (not IsKeyAvailable) then
    begin
      FSettings.Left := Left;
      FSettings.Top  := Top;
      FSettings.Width:= Width;
      FSettings.Height := Height;
      FSettings.ROH  := true;
      FSettings.ScanFiles := true;
      FSettings.Summ  := true;
      FSettings.Author := true;
      FSettings.Legend := true;
      SaveSettings;
    end
  else
    begin
      LoadSettings;
      Data.ROH    := FSettings.ROH;
      Data.ScanFiles := FSettings.ScanFiles;
      Left        := FSettings.Left;
      Top         := FSettings.Top;
      Width       := FSettings.Width;
      Height      := FSettings.Height;
    end;
end;

procedure TformMenuPrincipal.OptionsExecute(Sender: TObject);
begin
  ShowOptions(FSettings);
end;

procedure TformMenuPrincipal.ExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TformMenuPrincipal.AboutExecute(Sender: TObject);
begin
  ShowAbout;
end;

procedure TformMenuPrincipal.sbMainDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel;
  const Rect: TRect);
begin
  DrawIconEx(StatusBar.Canvas.Handle,Rect.Left,Rect.Top,Icon.Handle,16,16,0,0,DI_NORMAL);
end;

end.
